// MIDI呼吸控制器 by x7890
#include <MIDIUSB.h> // MIDIUSB库：https://www.arduino.cc/en/Reference/MIDIUSB
#define AD_SCK    15 // 时钟引脚
#define AD_DAT    14 // 数据引脚
#define AD_MIN  1000 // 最小值
#define AD_MAX 12000 // 最大值
#define MIDI_CH    0 // MIDI通道号
#define MIDI_CC   11 // MIDI控制器号

void setup() {
  Serial.begin(115200);
  Serial.setTimeout(10); // 防止未打开串口监视器时MIDI信号阻塞
  pinMode(AD_SCK, OUTPUT);
  pinMode(AD_DAT, INPUT_PULLUP);
  digitalWrite(AD_SCK, HIGH);
  delay(1); // 复位
  digitalWrite(AD_SCK, LOW);
  AD_read(); // 设置转换模式
}

int val_last = 0;
void loop() {
  int16_t ad = AD_read();
  int val = map(ad, AD_MIN, AD_MAX, 0, 127);
  val = constrain(val, 0, 127);
  if (val != val_last) {
    midiEventPacket_t event = {0x0B, 0xB0 | MIDI_CH, MIDI_CC, (byte)val};
    MidiUSB.sendMIDI(event);
    MidiUSB.flush();
    val_last = val;
  }
  Serial.print(ad);
  Serial.print(' ');
  Serial.print(map(val, 0, 127, AD_MIN, AD_MAX));
  Serial.print(' ');
  Serial.println(val);
}

int16_t AD_read() {
  while (digitalRead(AD_DAT))
    ;
  uint32_t data = 0;
  // 读取24个数据bit
  for (int i = 0; i < 24; i++) {
    digitalWrite(AD_SCK, HIGH);
    delayMicroseconds(1);
    data = (data << 1) | digitalRead(AD_DAT);
    digitalWrite(AD_SCK, LOW);
    delayMicroseconds(1);
  }
  // 下一次转换：差分输入，增益128，40Hz
  for (int i = 0; i < 3; i++) {
    digitalWrite(AD_SCK, HIGH);
    delayMicroseconds(1);
    digitalWrite(AD_SCK, LOW);
    delayMicroseconds(1);
  }
  return data >> 8; // 有符号数
}
